import logging

from saruman.app.queue import queue

import saruman.tasks.kernel.modprobe

__all__ = ['check']

logger = logging.getLogger(__name__)


def start_all():
    logger.warn("Chargement du module kernel dummy")
    log_pref = "Module Kernel `dummy` - "
    logger.info(log_pref + 'Check si le module est chargé')
    dummy_check_result = saruman.tasks.kernel.modprobe.Check().delay('dummy')
    logger.info(log_pref + 'La commande de check a retourné: {}'.format(dummy_check_result.get()))
    logger.info(log_pref + 'Le module {} activé'.format("est" if dummy_check_result.get() else "n'est pas"))
    if not dummy_check_result.get():
        logger.info(log_pref + 'Activation du module')
        dummy_active_result = saruman.tasks.kernel.modprobe.Add().delay('dummy', '')
        if dummy_active_result.get() is not None:
            logger.info(log_pref + 'La commande a retourné: `{}`'.format(dummy_active_result.get()))
        else:
            logger.error(log_pref + 'La commande a echoué ! Regarde les logs workers')
    logger.warn("Chargement du module kernel bridge")
    log_pref = "Module Kernel `bridge` - "
    logger.info(log_pref + 'Check si le module est chargé')
    bridge_check_result = saruman.tasks.kernel.modprobe.Check().delay('bridge')
    logger.info(log_pref + 'La commande de check a retourné: {}'.format(bridge_check_result.get()))
    logger.info(log_pref + 'Le module {} activé'.format("est" if bridge_check_result.get() else "n'est pas"))
    if not bridge_check_result.get():
        logger.info(log_pref + 'Activation du module')
        bridge_active_result = saruman.tasks.kernel.modprobe.Add().delay('bridge', '')
        if bridge_active_result.get() is not None:
            logger.info(log_pref + 'La commande a retourné: `{}`'.format(bridge_active_result.get()))
        else:
            logger.error(log_pref + 'La commande a echoué ! Regarde les logs workers')

if __name__ == '__main__':
    start_all()



CELERY_TASK_SERIALIZER = 'json'
CELERY_ACCEPT_CONTENT = ['json']
CELERY_RESULT_SERIALIZER = 'json'
CELERY_TIMEZONE = 'Europe/Paris'
CELERY_ENABLE_UTC = True
CELERY_RESULT_PERSISTENT = True
CELERY_EAGER_PROPAGATES_EXCEPTIONS = False



#!/usr/bin/env python3
from traceback import format_tb

import click
import pprint
import time
from colorlog import ColoredFormatter
import celery.bin.multi

from saruman.app.queue import queue
from saruman.actions.start import start_all

import logging
logger = logging.getLogger()
logger.setLevel(logging.WARN)

stream_handler = logging.StreamHandler()
formatter = ColoredFormatter(
        "%(log_color)s%(message)s",
        datefmt=None,
        reset=True,
        log_colors={
                'DEBUG':    'cyan',
                'INFO':     'green',
                'WARNING':  'yellow',
                'ERROR':    'red',
                'CRITICAL': 'red',
        }
)
stream_handler.setFormatter(formatter)
logger.addHandler(stream_handler)


__all__ = ['cli']


@click.group()
@click.help_option(help="Affiche ce message")
@click.version_option(version='1.0.0', help="Affiche la version")
def cli():
    pass


@cli.group(short_help="( enable | disable) Active ou desactive les workers")
@click.help_option(help="Affiche ce message")
def worker():
    pass


@worker.command(short_help="Active les workers", help="Affiche ce message")
@click.help_option(help="Affiche ce message")
@click.option('--verbose', is_flag=True, default=False, help="Augmente la verbosité")
def enable(verbose=False):
    click.secho('* Vérification de l\'état des worker', fg='blue', bold=True)
    heartbeat = queue.control.inspect().ping()
    if verbose:
        click.secho(pprint.pformat(heartbeat,  indent=4))
    if heartbeat is None:
        click.secho('  |> Les workers celery ne sont pas encore lancés', fg='yellow', bold=True)
        click.secho('* Lancement des workers', fg='blue', bold=True)
        multi = celery.bin.multi.MultiTool(quiet=True, nosplash=True)
        multi.start(['w1', '-A', 'firewall.app.queue', '-l', 'info'], cmd='celery worker')
        click.secho('* Attente du lancement des workers', fg='blue', bold=True)
        t_start = time.time()
        t_diff = 0
        while heartbeat is None:
            heartbeat = queue.control.inspect().ping()
            if heartbeat is not None:
                break
            else:
                time.sleep(0.5)
                t_diff = time.time() - t_start
                if verbose:
                    click.secho('  |> ' + str(t_diff) + 's')
        click.secho('  |> Les workers celery sont lancés', fg='green', bold=True)
    else:
        click.secho('  |> Les workers celery sont lancés', fg='green', bold=True)
        if verbose:
            click.secho(pprint.pformat(queue.control.inspect().stats(), indent=4))


@worker.command(short_help="Desactive les workers")
@click.help_option(help="Affiche ce message")
@click.option('--verbose', is_flag=True, default=False, help="Augmente la verbosité")
def disable(verbose=True):
    click.secho('* Fermeture des workers', fg='blue', bold=True)
    multi = celery.bin.multi.MultiTool(quiet=True, nosplash=True)
    multi.stop(['w1', '-A', 'firewall.app.queue', '-l', 'info'], cmd='celery worker')
    click.secho('* Attente de la fermeture des workers', fg='blue', bold=True)
    heartbeat = queue.control.inspect().ping()
    t_start = time.time()
    t_diff = 0
    while heartbeat is not None:
        heartbeat = queue.control.inspect().ping()
        if heartbeat is not None:
            break
        else:
            time.sleep(0.5)
            t_diff = time.time() - t_start
            if verbose:
                click.secho('  |> ' + str(t_diff) + 's')
    click.secho('  |> Les workers celery sont fermés', fg='green', bold=True)


@worker.command(short_help="Reload les workers", help="Affiche ce message")
@click.help_option(help="Affiche ce message")
@click.option('--verbose', is_flag=True, default=False, help="Augmente la verbosité")
def reload(verbose=False):
    click.secho('* Vérification de l\'état des worker', fg='blue', bold=True)
    heartbeat = queue.control.inspect().ping()
    if verbose:
        click.secho(pprint.pformat(heartbeat,  indent=4))
    if heartbeat is not None:
        click.secho('  |> Les workers celery sont lancés', fg='green', bold=True)
        if verbose:
            click.secho(pprint.pformat(queue.control.inspect().stats(), indent=4))
        click.secho('* Redémarage des workers', fg='blue', bold=True)
        multi = celery.bin.multi.MultiTool(quiet=True, nosplash=True)
        multi.restart(['w1', '-A', 'firewall.app.queue', '-l', 'info'], cmd='celery worker')
    else:
        click.secho('  |> Les workers celery ne sont pas encore lancés', fg='yellow', bold=True)
        click.secho('* Lancement des workers', fg='blue', bold=True)
        multi = celery.bin.multi.MultiTool(quiet=True, nosplash=True)
        multi.start(['w1', '-A', 'firewall.app.queue', '-l', 'info'], cmd='celery worker')
    click.secho('* Attente du lancement des workers', fg='blue', bold=True)
    t_start = time.time()
    t_diff = 0
    while heartbeat is None:
        heartbeat = queue.control.inspect().ping()
        if heartbeat is not None:
            break
        else:
            time.sleep(0.5)
            t_diff = time.time() - t_start
            if verbose:
                click.secho('  |> ' + str(t_diff) + 's')
    click.secho('  |> Les workers celery sont lancés', fg='green', bold=True)


@cli.command(short_help="Démarre le firewall")
@click.help_option(help="Affiche ce message")
@click.option('--verbose', is_flag=True, default=False, help="Augmente la verbosité")
def start(verbose):
    start_all()


if __name__ == '__main__':
    cli()



import celery

queue = celery.Celery("firewall",
                      backend='amqp://guest@localhost//',
                      broker='amqp://guest@localhost//',
                      include=[
                          "firewall.tasks",
                          "firewall.tasks.kernel",
                          "firewall.tasks.firewall",
                      ])

queue.config_from_object("firewall.conf.celery")

if __name__ == '__main__':
    pass



from celery import Task
from celery.utils.log import get_task_logger

# noinspection PyUnresolvedReferences
from plumbum.cmd import lsmod, modprobe

from saruman.helpers.error_handling import error_handling
from saruman.helpers.exceptions import FirewallNotAllowedError

__all__ = ['check']

logger = get_task_logger(__name__)


class Bridge(Task):
    name = 'kernel.modules.check'

    @error_handling(logger)
    def run(self, bridge_name, interface_name):
        logger.info("Création d'un nouveau bridge `{}`".format(bridge_name))
        logger.debug("Execution de ip")



from celery.utils.log import get_task_logger

from saruman.app.queue import queue

logger = get_task_logger(__name__)


@queue.task(name='misc.hello')
def hello(x):
    logger.info("It is a ...")
    logger.info(x)



from celery import Task
from celery.utils.log import get_task_logger

# noinspection PyUnresolvedReferences
from plumbum.cmd import lsmod, modprobe

from saruman.helpers.error_handling import error_handling
from saruman.helpers.exceptions import FirewallNotAllowedError

__all__ = ['check']

logger = get_task_logger(__name__)

allowed_mod = ['dummy']


class Check(Task):
    """
    Tache de vérification de l'activation d'un module dans le kernel

    Vérifie si le module `module_name` est activé dans le kernel.
    Se réfère à une liste des modules autorisés (ainsi, l'utilisateur ne peut pas
    supprimer le module du filesystem par exemple).
    La tache tourne dans un context (:py:func:error_handling) qui gère les erreurs

    """
    name = 'kernel.modules.check'

    def run(self, module_name):
        """
        :param  str module_name: le nom du module à checker
        :return: oui si le module est activé, non sinon
        :rtype: bool
        """
        with error_handling(logger):
            logger.info("Check pour voir si le module `{}` du kernel est activé".format(module_name))
            if module_name not in allowed_mod:
                raise FirewallNotAllowedError("Le module `{}` du kernel n'est pas whitelisté".format(module_name))
            logger.debug("Execution de lsmod")
            mod = lsmod()
            return module_name in mod


class Add(Task):
    """
    Tache de vérification de l'activation d'un module dans le kernel

    Vérifie si le module `module_name` est activé dans le kernel.
    Se réfère à une liste des modules autorisés (ainsi, l'utilisateur ne peut pas
    supprimer le module du filesystem par exemple).
    La tache tourne dans un context (:py:func:error_handling) qui gère les erreurs
    """
    name = 'kernel.modules.add'

    def run(self, module_name, module_args, **kwargs):
        """
        :param str module_name: le nom du module à checker
        :param dict module_args: un dictionnaire d'arguments
        """
        with error_handling(logger):
            logger.info("Activation du module `{}` du kernel".format(module_name))
            if module_name not in allowed_mod:
                raise FirewallNotAllowedError("Le module `{}` du kernel n'est pas whitelisté".format(module_name))
            logger.debug("Execution de modprobe")
            if module_args is not None:
                arguments = ['{}={}'.format(k, v) for k, v in module_args.items()]
                arguments = [module_name].extend(arguments)
                result = modprobe(arguments)
            else:
                result = modprobe[module_name]()
            logger.info("Le module `{}` du kernel est {}".format(module_name, "activé" if result else "désactivé"))


class Remove(Task):
    name = 'kernel.modules.remove'

    def run(self, module_name,  **kwargs):
        with error_handling(logger):
            logger.info("Desactivation du module `{}` du kernel".format(module_name))
            if module_name not in allowed_mod:
                raise FirewallNotAllowedError("Le module `{}` du kernel n'est pas whitelisté".format(module_name))
            logger.debug("Execution de modprobe")
            result = modprobe["-r", module_name]()
            logger.info("Le module `{}` du kernel est {}".format(module_name, "activé" if result else "désactivé"))



import contextlib
import logging

from plumbum.commands.processes import ProcessExecutionError, CommandNotFound, ProcessTimedOut
from celery.exceptions import (SecurityError, Ignore, QueueNotFound, WorkerShutdown, WorkerTerminate,
                               ImproperlyConfigured, NotRegistered, AlreadyRegistered,
                               TimeoutError, MaxRetriesExceededError, Retry,
                               TaskRevokedError, NotConfigured, AlwaysEagerIgnored,
                               InvalidTaskError, ChordError, CPendingDeprecationWarning,
                               CDeprecationWarning, FixupWarning, DuplicateNodenameWarning,
                               SoftTimeLimitExceeded, TimeLimitExceeded, WorkerLostError,
                               Terminated)
from saruman.helpers.exceptions import FirewallGenericError

__all__ = ['celery_error_handling']

WARNING = (Ignore, AlreadyRegistered, TimeoutError, CPendingDeprecationWarning,
           CDeprecationWarning, FixupWarning, DuplicateNodenameWarning,
           SoftTimeLimitExceeded, )
HANDLED_ERROR = ()
UNHANDLED_ERROR = (ProcessTimedOut, WorkerShutdown, WorkerTerminate, ImproperlyConfigured, NotRegistered,
                   MaxRetriesExceededError, TaskRevokedError, NotConfigured, AlwaysEagerIgnored, InvalidTaskError,
                   ChordError, FirewallGenericError, TimeLimitExceeded, WorkerLostError, Terminated)
CRITICAL = (ProcessExecutionError, CommandNotFound, SecurityError, QueueNotFound)


@contextlib.contextmanager
def error_handling(logger):
    try:
        assert isinstance(logger, logging.Logger)
        try:
            yield
        except WARNING as e:
            logger.warning(e)
        except UNHANDLED_ERROR as e:
            logger.error(e)
            raise
        except HANDLED_ERROR as e:
            logger.error(e)
        except CRITICAL as e:
            logger.critical(e)
    except Exception as e:
        print(e)



def get_celery_worker_status():
    error_key = "ERROR"
    try:
        # noinspection PyUnresolvedReferences
        from celery.task.control import inspect
        inspection = inspect()
        d = inspection.stats()
        if not d:
            d = {error_key: 'No running Celery workers were found.'}
    except IOError as e:
        from errno import errorcode
        msg = "Error connecting to the backend: " + str(e)
        if len(e.args) > 0 and errorcode.get(e.args[0]) == 'ECONNREFUSED':
            msg += ' Check that the RabbitMQ server is running.'
        d = {error_key: msg}
    except ImportError as e:
        d = {error_key: str(e)}
    return d

if __name__ == '__main__':
    get_celery_worker_status()



#!/usr/bin/env python3
from setuptools import setup, find_packages
from setuptools.command.install import install as _install


def read(filename):
    try:
        with open(filename) as f:
            return f.read()
    except NameError:
        with open(filename, 'r') as f:
            return f.read()

long_description = u'\n\n'.join([read('README.rst'),
                                 read('CREDITS.rst'),
                                 read('CHANGES.rst')])


class Install(_install):
    def run(self):
        _install.run(self)
        print("Post Install")

setup(
        name='saruman',
        version='0.2.0',
        packages=find_packages(),
        url='https://github.com/tychota/saruman',
        license='MIT',
        author='tychota',
        author_email='tycho.tatitscheff+saruman@gadz.org',
        description='A firewall that leverage AMQP workqueue ! Build by iresam for iresam !',
        long_description=long_description,
        install_requires=[
            'celery',
            'plumbum',
            'click',
            'colorlog',
            'pyyaml'
        ],
        setup_requires=['pytest-runner'],
        tests_require=['pytest'],
        test_suite="tests",
        entry_points='''
            [console_scripts]
            saruman=saruman.__main__:cli
        ''',
        cmdclass={'install': Install},
        zip_safe=False,
        keywords=['firewall', 'amqp', 'nftables', 'dhcp, reverse-proxy'],
        classifiers=[
            'Development Status :: 2 - Pre-Alpha',
            'Environment :: Console',
            'Intended Audience :: System Administrators',
            'Natural Language :: English',
            'Natural Language :: French',
            'Operating System :: POSIX :: Linux',
            'Topic :: Internet :: Name Service (DNS)',
            'Topic :: Internet :: Proxy Servers',
            'Topic :: System :: Networking',
            'Topic :: System :: Networking :: Firewalls',
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
        ],
)
